from django.db import migrations


def migrate_repository_rights(apps, schema):
    Right = apps.get_model("users", "Right")
    Repository = apps.get_model("process", "Repository")
    Worker = apps.get_model("process", "Worker")
    ContentType = apps.get_model("contenttypes", "ContentType")

    repository_type = ContentType.objects.get_for_model(Repository)
    worker_type = ContentType.objects.get_for_model(Worker)
    repository_rights = Right.objects.filter(content_type=repository_type)

    to_update = []
    to_create = []
    for repo_right in repository_rights:
        repository_id = repo_right.content_id
        repository_workers = Worker.objects.filter(repository_id=repository_id)
        for worker in repository_workers:
            right = Right.objects.filter(
                content_type=worker_type,
                content_id=worker.id,
                user_id=repo_right.user_id,
                group_id=repo_right.group_id,
            ).first()
            # A right on the worker already exists for this user or group
            if right and right.level < repo_right.level:
                # If the current access level < the one defined at repository level, update it
                right.level = repo_right.level
                to_update.append(right.id)
            # Otherwise create a new right on the worker for this user or group
            elif not right:
                to_create.append(
                    Right(
                        content_id=worker.id,
                        user_id=repo_right.user_id,
                        group_id=repo_right.group_id,
                        level=repo_right.level,
                        content_type=worker_type
                    )
                )
    Right.objects.bulk_update(to_update, fields=["level"], batch_size=1000)
    Right.objects.bulk_create(to_create, batch_size=1000)


class Migration(migrations.Migration):

    dependencies = [
        ("process", "0037_workerrun_use_gpu"),
    ]

    operations = [
        migrations.RunPython(
            migrate_repository_rights,
            elidable=True,
        ),
    ]
